import unittest
from heaserver.service.db import awss3bucketobjectkey


class BucketObjectKeyTest(unittest.TestCase):
    def test_replace_parent_folder(self):
        self.assertEqual('baz/bar', awss3bucketobjectkey.replace_parent_folder('foo/oof/bar', 'baz/bar'))


# joined, split
split_join_data = [
    ('hello/goodbye/foobar/', ('hello/goodbye/', 'foobar/')),
    ('hello/goodbye/foobar', ('hello/goodbye/', 'foobar')),
    ('foobar', ('', 'foobar')),
    ('/', ('', '/'))
]


def join_test_generator(joined, split):
    def test(self):
        self.assertEquals(joined, awss3bucketobjectkey.join(*split))
    return test


def split_test_generator(joined, split):
    def test(self):
        self.assertEquals(split, awss3bucketobjectkey.split(joined))
    return test


for d in split_join_data:
    join_test = join_test_generator(d[0], d[1])
    setattr(BucketObjectKeyTest, f'test_join_{d[1]}', join_test)
    split_test = split_test_generator(d[0], d[1])
    setattr(BucketObjectKeyTest, f'test_split_{d[0]}', split_test)


# original, encoded, is_folder
encode_decode_data = [('foobar', 'Zm9vYmFy', False),
                      ('foobar/', 'Zm9vYmFyLw==', True),
                      ('hello/goodbye/foobar/', 'aGVsbG8vZ29vZGJ5ZS9mb29iYXIv', True)]


def encode_test_generator(orig, encoded):
    def test(self):
        self.assertEquals(encoded, awss3bucketobjectkey.encode_key(orig))
    return test


def decode_test_generator(orig, encoded):
    def test(self):
        self.assertEquals(orig, awss3bucketobjectkey.decode_key(encoded))
    return test


def is_folder_test_generator(orig, is_folder):
    def test(self):
        self.assertEqual(is_folder, awss3bucketobjectkey.is_folder(orig))
    return test


for d in encode_decode_data:
    join_test = encode_test_generator(d[0], d[1])
    setattr(BucketObjectKeyTest, f'test_encode_{d[0]}', join_test)
    split_test = decode_test_generator(d[0], d[1])
    setattr(BucketObjectKeyTest, f'test_decode_{d[0]}', split_test)
    is_folder_test = is_folder_test_generator(d[0], d[2])
    setattr(BucketObjectKeyTest, f'test_is_folder_{d[0]}', is_folder_test)




from unittest import IsolatedAsyncioTestCase
from .awss3foldertestcase import AWSS3FolderTestCase
from heaserver.service.db import awsservicelib
from heaserver.service.db.aws import S3
import boto3
from botocore.exceptions import ClientError
from freezegun.api import FakeDatetime
from dateutil.tz import tzutc
from heaobject.user import NONE_USER
from moto import mock_sts


class TestS3GetAccount(IsolatedAsyncioTestCase):

    @mock_sts()
    def run(self, result):
        super().run(result)

    async def test_get_account(self):
        sts = boto3.client('sts')
        actual = await S3._get_account(sts, NONE_USER)
        self.assertEquals('123456789012', actual.id)


class TestAWSServiceLib(AWSS3FolderTestCase):

    @mock_sts()
    def run(self, result):
        super().run(result)

    async def test_get_property(self):
        async with self.client.request('GET', '/properties/CLOUD_AWS_CRED_URL') as resp:
            self.assertEquals(200, resp.status)

    async def test_get_property_not_found(self):
        async with self.client.request('GET', '/properties/TEST') as resp:
            self.assertEquals(404, resp.status)

    async def test_get_volume_id_for_account_id(self):
        async with self.client.request('GET', '/accounts/123456789012') as resp:
            self.assertEquals(200, resp.status)

    async def test_list_bucket_not_found(self):
        s3 = boto3.client('s3')
        with self.assertRaises(ClientError):
            l = [o async for o in awsservicelib.list_objects(s3, 'blah')]

    async def test_list_bucket(self):
        s3 = boto3.client('s3')
        expected = [{'Key': 'TestFolder/', 'LastModified': FakeDatetime(2022, 5, 17, 0, 0, tzinfo=tzutc()),
                     'ETag': '"d41d8cd98f00b204e9800998ecf8427e"', 'Size': 0, 'StorageClass': 'STANDARD'},
                    {'Key': 'TestFolder2/', 'LastModified': FakeDatetime(2022, 5, 17, 0, 0, tzinfo=tzutc()),
                     'ETag': '"d41d8cd98f00b204e9800998ecf8427e"', 'Size': 0, 'StorageClass': 'STANDARD'}]
        actual = [o async for o in awsservicelib.list_objects(s3, 'arp-scale-2-cloud-bucket-with-tags11')]
        self.assertEquals(expected, actual)

    async def test_list_empty_bucket_with_filter(self):
        s3 = boto3.client('s3')
        expected = [{'Key': 'TestFolder/', 'LastModified': FakeDatetime(2022, 5, 17, 0, 0, tzinfo=tzutc()),
                     'ETag': '"d41d8cd98f00b204e9800998ecf8427e"', 'Size': 0, 'StorageClass': 'STANDARD'},
                    {'Key': 'TestFolder2/', 'LastModified': FakeDatetime(2022, 5, 17, 0, 0, tzinfo=tzutc()),
                     'ETag': '"d41d8cd98f00b204e9800998ecf8427e"', 'Size': 0, 'StorageClass': 'STANDARD'}]
        actual = [o async for o in
                  awsservicelib.list_objects(s3, 'arp-scale-2-cloud-bucket-with-tags11', prefix='TestFolder')]
        self.assertEquals(expected, actual)

    async def test_list_empty_bucket_with_filter_one(self):
        s3 = boto3.client('s3')
        expected = [{'Key': 'TestFolder/', 'LastModified': FakeDatetime(2022, 5, 17, 0, 0, tzinfo=tzutc()),
                     'ETag': '"d41d8cd98f00b204e9800998ecf8427e"', 'Size': 0, 'StorageClass': 'STANDARD'}]
        actual = [o async for o in
                  awsservicelib.list_objects(s3, 'arp-scale-2-cloud-bucket-with-tags11', prefix='TestFolder/')]
        self.assertEquals(expected, actual)

    async def test_delete_object(self):
        s3 = boto3.client('s3')
        actual = await awsservicelib._delete_object(s3, bucket_name='arp-scale-2-cloud-bucket-with-tags11',
                                                    key='TestFolder2/', recursive=True)
        self.assertEquals(204, actual.status)

    async def test_delete_object_not_found(self):
        s3 = boto3.client('s3')
        actual = await awsservicelib._delete_object(s3, bucket_name='arp-scale-2-cloud-bucket-with-tags11',
                                                    key='foobar/', recursive=True)
        self.assertEquals(404, actual.status)

    async def test_delete_object_bucket_not_found(self):
        s3 = boto3.client('s3')
        actual = await awsservicelib._delete_object(s3, bucket_name='arp-scale-2-cloud-bucket-with-tags1',
                                                    key='TestFolder2/', recursive=True)
        self.assertEquals(404, actual.status)

    async def test_copy_object(self):
        s3 = boto3.client('s3')
        actual = await awsservicelib._copy_object(s3, source_bucket_name='arp-scale-2-cloud-bucket-with-tags11',
                                                  source_key_name='TestFolder2/',
                                                  destination_bucket_name='arp-scale-2-cloud-bucket-with-tags11',
                                                  destination_key_name='foobar/TestFolder3/')
        self.assertEquals(201, actual.status)

    async def test_copy_object_not_found_source_bucket(self):
        s3 = boto3.client('s3')
        actual = await awsservicelib._copy_object(s3, source_bucket_name='arp-scale-2-cloud-bucket-with-tags1',
                                                  source_key_name='TestFolder2/',
                                                  destination_bucket_name='arp-scale-2-cloud-bucket-with-tags11',
                                                  destination_key_name='foobar/TestFolder3/')
        self.assertEquals(400, actual.status)

    async def test_copy_object_not_found_destination_bucket(self):
        s3 = boto3.client('s3')
        actual = await awsservicelib._copy_object(s3, source_bucket_name='arp-scale-2-cloud-bucket-with-tags11',
                                                  source_key_name='TestFolder2/',
                                                  destination_bucket_name='arp-scale-2-cloud-bucket-with-tags1',
                                                  destination_key_name='foobar/TestFolder3/')
        self.assertEquals(400, actual.status)

    async def test_copy_object_not_found_object(self):
        s3 = boto3.client('s3')
        actual = await awsservicelib._copy_object(s3, source_bucket_name='arp-scale-2-cloud-bucket-with-tags11',
                                                  source_key_name='TestFolder22/',
                                                  destination_bucket_name='arp-scale-2-cloud-bucket-with-tags11',
                                                  destination_key_name='foobar/TestFolder3/')
        self.assertEquals(400, actual.status)



from unittest import TestCase
from heaserver.service.db import awsservicelib
from botocore.exceptions import ClientError


class TestAWSServiceLib(TestCase):
    def test_key_in_folder(self):
        self.assertTrue(awsservicelib.key_in_folder('foo/bar/baz', 'foo/bar/'))

    def test_key_in_non_folder(self):
        self.assertFalse(awsservicelib.key_in_folder('foo/bar/baz', 'foo/bar'))

    def test_key_not_in_folder(self):
        self.assertFalse(awsservicelib.key_in_folder('foo/bar', 'foo/bar/baz/'))

    def test_decode_folder_root(self):
        self.assertEqual('', awsservicelib.decode_folder('root'))

    def test_decode_folder_non_root(self):
        self.assertEqual('/', awsservicelib.decode_folder('Lw=='))

    def test_decode_not_folder(self):
        self.assertEqual(None, awsservicelib.decode_folder('VGV4dEZpbGUucGRm'))

    def test_handle_client_error_404(self):
        c = ClientError(error_response={'Error': {'Code': awsservicelib.CLIENT_ERROR_404}}, operation_name='foo')
        self.assertEqual(404, awsservicelib.handle_client_error(c).status)

    def test_handle_client_error_no_such_bucket(self):
        c = ClientError(error_response={'Error': {'Code': awsservicelib.CLIENT_ERROR_NO_SUCH_BUCKET}}, operation_name='foo')
        self.assertEqual(404, awsservicelib.handle_client_error(c).status)

    def test_handle_client_error_unknown(self):
        c = ClientError(error_response={'Error': {'Code': "It's wrecked"}},
                        operation_name='foo')
        self.assertEqual(500, awsservicelib.handle_client_error(c).status)


